=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2007-03-20 13:49:49 +0000
+++ bzrlib/info.py	2007-03-20 15:09:49 +0000
@@ -322,6 +322,52 @@
     _show_repository_stats(stats)
 
 
+def describe_layout(repository=None, branch=None, tree=None):
+    """Convert a control directory layout into a user-understandable term
+
+    Common outputs include "Standalone tree", "Repository branch" and
+    "Checkout".  Uncommon outputs include "Unshared repository with trees"
+    and "Empty control directory"
+    """
+    if repository is None:
+        return 'Empty control directory'
+    if branch is None and tree is None:
+        if repository.is_shared():
+            phrase = 'Shared repository'
+        else:
+            phrase = 'Unshared repository'
+        if repository.make_working_trees():
+            phrase += ' with trees'
+        return phrase
+    else:
+        if repository.is_shared():
+            independence = "Repository "
+        else:
+            independence = "Standalone "
+        if tree is not None:
+            phrase = "tree"
+        else:
+            phrase = "branch"
+        if branch is None and tree is not None:
+            phrase = "branchless tree"
+        else:
+            if (tree is not None and tree.bzrdir.root_transport.base !=
+                branch.bzrdir.root_transport.base):
+                if independence == 'Standalone ':
+                    independence = ''
+                phrase = "Lightweight checkout"
+            elif branch.get_bound_location() is not None:
+                if independence == 'Standalone ':
+                    independence = ''
+                if tree is None:
+                    phrase = "Bound branch"
+                else:
+                    phrase = "Checkout"
+        if independence != "":
+            phrase = phrase.lower()
+        return "%s%s" % (independence, phrase)
+
+
 @deprecated_function(zero_eight)
 def show_info(b):
     """Please see show_bzrdir_info."""

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-03-15 22:35:35 +0000
+++ bzrlib/tests/__init__.py	2007-03-20 15:09:49 +0000
@@ -1961,6 +1961,7 @@
                    'bzrlib.tests.test_https_ca_bundle',
                    'bzrlib.tests.test_identitymap',
                    'bzrlib.tests.test_ignores',
+                   'bzrlib.tests.test_info',
                    'bzrlib.tests.test_inv',
                    'bzrlib.tests.test_knit',
                    'bzrlib.tests.test_lazy_import',

=== added file 'bzrlib/tests/test_info.py'
--- bzrlib/tests/test_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_info.py	2007-03-20 15:09:49 +0000
@@ -0,0 +1,56 @@
+from bzrlib import (
+    bzrdir,
+    info,
+    tests,
+    )
+
+
+class TestInfo(tests.TestCaseWithTransport):
+
+    def test_describe_standalone_layout(self):
+        tree = self.make_branch_and_tree('tree')
+        self.assertEqual('Empty control directory', info.describe_layout())
+        self.assertEqual('Unshared repository with trees',
+            info.describe_layout(tree.branch.repository))
+        tree.branch.repository.set_make_working_trees(False)
+        self.assertEqual('Unshared repository',
+            info.describe_layout(tree.branch.repository))
+        self.assertEqual('Standalone branch',
+            info.describe_layout(tree.branch.repository, tree.branch))
+        self.assertEqual('Standalone branchless tree',
+            info.describe_layout(tree.branch.repository, None, tree))
+        self.assertEqual('Standalone tree',
+            info.describe_layout(tree.branch.repository, tree.branch, tree))
+        tree.branch.bind(tree.branch)
+        self.assertEqual('Bound branch',
+            info.describe_layout(tree.branch.repository, tree.branch))
+        self.assertEqual('Checkout',
+            info.describe_layout(tree.branch.repository, tree.branch, tree))
+        checkout = tree.branch.create_checkout('checkout', lightweight=True)
+        self.assertEqual('Lightweight checkout',
+            info.describe_layout(checkout.branch.repository, checkout.branch,
+                                 checkout))
+
+
+    def test_describe_repository_layout(self):
+        repository = self.make_repository('.', shared=True)
+        tree = bzrdir.BzrDir.create_branch_convenience('tree',
+            force_new_tree=True).bzrdir.open_workingtree()
+        self.assertEqual('Shared repository with trees',
+            info.describe_layout(tree.branch.repository))
+        repository.set_make_working_trees(False)
+        self.assertEqual('Shared repository',
+            info.describe_layout(tree.branch.repository))
+        self.assertEqual('Repository branch',
+            info.describe_layout(tree.branch.repository, tree.branch))
+        self.assertEqual('Repository branchless tree',
+            info.describe_layout(tree.branch.repository, None, tree))
+        self.assertEqual('Repository tree',
+            info.describe_layout(tree.branch.repository, tree.branch, tree))
+        tree.branch.bind(tree.branch)
+        self.assertEqual('Repository checkout',
+            info.describe_layout(tree.branch.repository, tree.branch, tree))
+        checkout = tree.branch.create_checkout('checkout', lightweight=True)
+        self.assertEqual('Repository lightweight checkout',
+            info.describe_layout(checkout.branch.repository, checkout.branch,
+                                 checkout))

